/***************************************************************************/
/** @file     about.cpp
  * @brief    The About tab page (implementation).
  *
  * @b begin      : Wed Dec 29 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <kapp.h>
#include <klocale.h>
#include <kiconloader.h>
#include <ktextbrowser.h>

#include <qlabel.h>
#include <qscrollview.h>
#include <qpushbutton.h>
#include <qframe.h>

#include "about.h"
#include "../config.h"

/*------------------------------------------------------------------------ */
About::About(QWidget *parent, const char *name) : QWidget(parent,name)
{
  const QString translator("<li><b>%1</b> (%2)</li>");
  
  this->resize(400,100);
  QLabel_1= new QLabel(this);
  QLabel_1->move(10,15);
  QLabel_1->setPixmap(DesktopIcon("ksetiwatch"));
  QLabel_1->resize(QLabel_1->sizeHint());

  Help = new QPushButton(this);
  Help->setMinimumSize(75, 25);
  Help->move(48,18);
  Help->setText(i18n("Help"));
  Help->resize(Help->sizeHint());
  connect(Help, SIGNAL(clicked()), SLOT(slotShowHelp()) );

  QLabel_2= new QLabel(this);
  QLabel_2->move(10,65);
  QLabel_2->setPixmap(DesktopIcon("MadeWithKDev"));
  QLabel_2->resize(QLabel_2->sizeHint());

  AboutView = new KTextBrowser(this);
  AboutView->setGeometry(48 + Help->size().width() + 7, 5, 490, 100);
  QString buffer = 
    i18n("This is <a href=\"http://ksetiwatch.sf.net\">Ksetiwatch</a> %1, developed by "
         "<a href=\"mailto:gmachel@users.sf.net\">Gordon Machel</a> "
         "(gmachel@users.sf.net).").arg(VERSION) +
    i18n("<p>Ksetiwatch monitors multiple "
         "<a href=\"http://setiathome.ssl.berkeley.edu/index.html\">SETI@home</a> "
         "instances running on your computer or in your network. It "
         "shares the same look-and-feel of the popular "
         "<a href=\"http://members.shaw.ca/mloukko/SETI.html\">SETIWatch</a> "
         "tool for the Windows platforms, written by Mark Loukko.</p>") +
    i18n("<p>To make full use of this program, the latest version of the SETI@home "
         "client is required. Command-line clients for various platforms "
         "can be found <a href=\"http://setiathome.ssl.berkeley.edu/unix.html\">here</a>.</p>") +
    i18n("<p>Ksetiwatch is published under the GNU Public License (GPL), version 2.</p>") +
    i18n("<p>Special thanks to <b>%1, %2, %3,</b> and <b>%4</b> for providing precompiled "
         "packages for various Linux distributions.</p>")
         .arg("John Aldrich")
         .arg("Renaud Michel")
         .arg("Stephan Windischmann")
         .arg("Diego Iastrubni") +
    i18n("<p>Ksetiwatch has been translated to many different languages. Credits for this "
         "work go to:</p>") +
    "<ul>" +
    translator.arg("Marco Wegner").arg(i18n("Coordination with the KDE i18n teams")) +
    translator.arg("Juan Manuel Garca Molina, Luis Bolinches Martinez").arg(i18n("Spanish")) +
    translator.arg("Renaud Michel").arg(i18n("French")) +
    translator.arg("Diego Iastrubni").arg(i18n("Hebrew")) +
    translator.arg("Marc").arg(i18n("German handbook")) +
    translator.arg("Tamas Szanto").arg(i18n("Hungarian")) +
    translator.arg("Stefan Asserhll").arg(i18n("Swedish")) +
    //translator.arg("Grkem etin").arg(i18n("Turkish")) +
    translator.arg("Albert Astals Cid").arg(i18n("Catalan")) +
    translator.arg("Pedro Morais").arg(i18n("Portuguese")) +
    translator.arg("Vladimir Vuksan").arg(i18n("Croatian")) +
    "</ul>" +
    i18n("<p>Many thanks for helpful hints, patches, and suggestions are given to: ") +
         "Frode Tenneb, Christian Srensen, Kenny Lim, Hans Motshrg, Roeland Th. Jansen, "
         "Matthew Soffen, Peter Drumm, Stefan Pfeiffer, Hugues Talbot, Jan Knutar, Pelon, "
         "Dan Crooks, Xet Erixon, " + i18n("and") + " Kjell Randa.</p>";
  AboutView->setText(buffer);
}

/*------------------------------------------------------------------------ */
About::~About()
{
}

/*------------------------------------------------------------------------ */
void About::resizeEvent(QResizeEvent*)
{
AboutView->resize(490, this->height()-10);
}

/*------------------------------------------------------------------------ */
void About::slotShowHelp()
{
kapp->invokeHelp();
}

#include "about.moc"

