/***************************************************************************/
/** @file     addsetilocdlg.cpp
  * @brief    A dialog for adding and editing a %S@h location
  *           (implementation).
  *
  * @b begin      : Sat Oct 30 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * @todo Check for correct %S@h directory when clicking OK, or when the
  *       corresponding edit field loses focus.
  **************************************************************************/
#include <kapp.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kcolorbutton.h>
#include <kiconloader.h>

#include <qcheckbox.h>
#include <qtooltip.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qslider.h>

#include "addsetilocdlg.h"

/*------------------------------------------------------------------------ */
AddSetiLocDlg::AddSetiLocDlg(QWidget *parent, const char *name)
             : KDialogBase(IconList, QString::null, Ok|Cancel, Ok, parent, name)
{
resize(400, 300);
enableButtonOK(false);

setupLocationPage();
setupClientPage();
setupLoggingPage();

QToolTip::add(StopAfterProcChkBox, i18n("Process the work unit and stop after \n"
                                   "processing is complete, do not return \n"
                                   "the result."));

QToolTip::add(StopAfterSendChkBox, i18n("Finish the current work unit, and \n"
                                   "return the result without downloading\n"
                                   "a new work unit."));

QToolTip::add(EmailChkBox, i18n("Send email (to login email address) on\n"
                           "errors. This option is not available\n"
                           "for all clients."));

QToolTip::add(GraphicsChkBox, i18n("Generate a data stream for the xsetiathome\n"
                              "graphical interface (see README.xsetiathome).\n"
                              "This option is not available for all clients."));


DirecButton->setPixmap(SmallIcon("fileopen"));
StartExeButton->setPixmap(SmallIcon("fileopen"));
StopExeButton->setPixmap(SmallIcon("fileopen"));
RedirButton->setPixmap(SmallIcon("fileopen"));
ColorButton->setColor(Qt::red);
}

/*------------------------------------------------------------------------ */
AddSetiLocDlg::~AddSetiLocDlg()
{
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setupLocationPage()
{
QFrame* page = addPage(i18n("Location"), i18n("General location settings"),
                       DesktopIcon("ksetiwatch", KIcon::SizeMedium));

LocationLayout = new QGridLayout( page );
LocationLayout->setSpacing( 6 );
LocationLayout->setMargin( 0 );

DescrEdit = new QLineEdit( page, "DescrEdit" );
DescrEdit->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, DescrEdit->sizePolicy().hasHeightForWidth() ) );
DescrEdit->setMinimumSize( QSize( 210, 0 ) );
QToolTip::add(  DescrEdit, i18n( "A descriptive name for the location." ) );

LocationLayout->addWidget( DescrEdit, 1, 0 );

ColorButton = new KColorButton( page, "ColorButton" );
ColorButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, ColorButton->sizePolicy().hasHeightForWidth() ) );
ColorButton->setMinimumSize( QSize( 60, 25 ) );
QToolTip::add(  ColorButton, i18n( "This color will be used for the markers on the skymap." ) );

LocationLayout->addWidget( ColorButton, 1, 1 );

DescrLabel = new QLabel( page, "DescrLabel" );
DescrLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, DescrLabel->sizePolicy().hasHeightForWidth() ) );
DescrLabel->setText( i18n( "Description:" ) );

LocationLayout->addWidget( DescrLabel, 0, 0 );

ColorLabel = new QLabel( page, "ColorLabel" );
ColorLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, ColorLabel->sizePolicy().hasHeightForWidth() ) );
ColorLabel->setText( i18n( "Color:" ) );

LocationLayout->addWidget( ColorLabel, 0, 1 );

LocationSubLayout1 = new QGridLayout;
LocationSubLayout1->setSpacing( 6 );
LocationSubLayout1->setMargin( 0 );

DirecLabel = new QLabel( page, "DirecLabel" );
DirecLabel->setText( i18n( "SETI@home directory:" ) );

LocationSubLayout1->addWidget( DirecLabel, 0, 0 );

DirecButton = new QPushButton( page, "DirecButton" );
DirecButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, DirecButton->sizePolicy().hasHeightForWidth() ) );
DirecButton->setMaximumSize( QSize( 40, 30 ) );

LocationSubLayout1->addWidget( DirecButton, 1, 1 );

DirecEdit = new QLineEdit( page, "DirecEdit" );
QToolTip::add(  DirecEdit, i18n( "The directory where the S@h files reside." ) );

LocationSubLayout1->addWidget( DirecEdit, 1, 0 );

LocationLayout->addMultiCellLayout( LocationSubLayout1, 2, 2, 0, 1 );

LocationSubLayout2 = new QGridLayout;
LocationSubLayout2->setSpacing( 6 );
LocationSubLayout2->setMargin( 0 );

StartExeEdit = new QLineEdit( page, "StartExeEdit" );
StartExeEdit->setEnabled( FALSE );
StartExeEdit->setText( i18n( "./setiathome" ) );
QToolTip::add(  StartExeEdit, i18n( "Enter here path and file name of the executable." ) );

LocationSubLayout2->addWidget( StartExeEdit, 1, 0 );

StartExeChkBox = new QCheckBox( page, "StartExeChkBox" );
StartExeChkBox->setText( i18n( "Start-up executable:" ) );
QToolTip::add(  StartExeChkBox, i18n( "Enable to run a customized executable at start-up." ) );

LocationSubLayout2->addWidget( StartExeChkBox, 0, 0 );

StartExeButton = new QPushButton( page, "StartExeButton" );
StartExeButton->setEnabled( FALSE );
StartExeButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, StartExeButton->sizePolicy().hasHeightForWidth() ) );
StartExeButton->setMaximumSize( QSize( 40, 30 ) );

LocationSubLayout2->addWidget( StartExeButton, 1, 1 );

LocationLayout->addMultiCellLayout( LocationSubLayout2, 3, 3, 0, 1 );

LocationSubLayout3 = new QGridLayout;
LocationSubLayout3->setSpacing( 6 );
LocationSubLayout3->setMargin( 0 );

StopExeEdit = new QLineEdit( page, "StopExeEdit" );
StopExeEdit->setEnabled( FALSE );
QToolTip::add(  StopExeEdit, i18n( "Enter here path and file name of the executable." ) );

LocationSubLayout3->addWidget( StopExeEdit, 1, 0 );

StopExeChkBox = new QCheckBox( page, "StopExeChkBox" );
StopExeChkBox->setText( i18n( "Shut-down executable:" ) );
QToolTip::add(  StopExeChkBox, i18n( "Enable to run a customized executable at shut-down." ) );

LocationSubLayout3->addWidget( StopExeChkBox, 0, 0 );

StopExeButton = new QPushButton( page, "StopExeButton" );
StopExeButton->setEnabled( FALSE );
StopExeButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, StopExeButton->sizePolicy().hasHeightForWidth() ) );
StopExeButton->setMaximumSize( QSize( 40, 30 ) );

LocationSubLayout3->addWidget( StopExeButton, 1, 1 );

LocationLayout->addMultiCellLayout( LocationSubLayout3, 4, 4, 0, 1 );
QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
LocationLayout->addItem( spacer, 5, 0 );

// signals and slots connections
connect( DirecButton, SIGNAL( clicked() ), this, SLOT( slotGetSetiDirectory() ) );
connect( DescrEdit, SIGNAL( textChanged(const QString&) ), this, SLOT( slotHandleText(const QString&) ) );
connect( DirecEdit, SIGNAL( textChanged(const QString&) ), this, SLOT( slotHandleText(const QString&) ) );
connect( StartExeChkBox, SIGNAL( toggled(bool) ), StartExeEdit, SLOT( setEnabled(bool) ) );
connect( StartExeChkBox, SIGNAL( toggled(bool) ), StartExeButton, SLOT( setEnabled(bool) ) );
connect( StopExeChkBox, SIGNAL( toggled(bool) ), StopExeEdit, SLOT( setEnabled(bool) ) );
connect( StopExeChkBox, SIGNAL( toggled(bool) ), StopExeButton, SLOT( setEnabled(bool) ) );
connect( StopExeButton, SIGNAL( clicked() ), this, SLOT( slotGetStopExe() ) );
connect( StartExeButton, SIGNAL( clicked() ), this, SLOT( slotGetStartExe() ) );

// tab order
setTabOrder( DescrEdit, DirecEdit );
setTabOrder( DirecEdit, DirecButton );
setTabOrder( DirecButton, StartExeChkBox );
setTabOrder( StartExeChkBox, StartExeEdit );
setTabOrder( StartExeEdit, StartExeButton );
setTabOrder( StartExeButton, StopExeChkBox );
setTabOrder( StopExeChkBox, StopExeEdit );
setTabOrder( StopExeEdit, StopExeButton );
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setupLoggingPage()
{
QFrame* page = addPage(i18n("WU Logging"), i18n("Logging of completed work units"),
                       DesktopIcon("document", KIcon::SizeMedium));

LoggingLayout = new QGridLayout( page );
LoggingLayout->setSpacing( 6 );
LoggingLayout->setMargin( 0 );

RedirEdit = new QLineEdit( page, "RedirEdit" );
QToolTip::add(  RedirEdit, i18n( "Log files will be stored in this folder." ) );

LoggingLayout->addWidget( RedirEdit, 2, 0 );

RedirButton = new QPushButton( page, "RedirButton" );
RedirButton->setMaximumSize( QSize( 40, 30 ) );

LoggingLayout->addWidget( RedirButton, 2, 1 );

RedirChkBox = new QCheckBox( page, "RedirChkBox" );
RedirChkBox->setText( i18n( "Redirect Log Files to:" ) );

LoggingLayout->addMultiCellWidget( RedirChkBox, 1, 1, 0, 1 );

LogChkBox = new QCheckBox( page, "LogChkBox" );
LogChkBox->setText( i18n( "Enable Logging of Work Units" ) );
LogChkBox->setChecked( TRUE );

LoggingLayout->addMultiCellWidget( LogChkBox, 0, 0, 0, 1 );
QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
LoggingLayout->addItem( spacer, 3, 0 );

// signals and slots connections
connect( LogChkBox, SIGNAL( toggled(bool) ), RedirChkBox, SLOT( setEnabled(bool) ) );
connect( LogChkBox, SIGNAL( toggled(bool) ), RedirEdit, SLOT( setEnabled(bool) ) );
connect( LogChkBox, SIGNAL( toggled(bool) ), RedirButton, SLOT( setEnabled(bool) ) );
connect( RedirButton, SIGNAL( clicked() ), this, SLOT( slotGetLogDirectory() ) );

// tab order
setTabOrder( LogChkBox, RedirChkBox );
setTabOrder( RedirChkBox, RedirEdit );
setTabOrder( RedirEdit, RedirButton );
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setupClientPage()
{
QFrame* page = addPage(i18n("SETI@home Client"), i18n("Settings of the SETI@home client"),
                       DesktopIcon("exec", KIcon::SizeMedium));

ClientLayout = new QVBoxLayout( page );
ClientLayout->setSpacing( 6 );
ClientLayout->setMargin( 0 );

StartSetiChkBox = new QCheckBox( page, "StartSetiChkBox" );
StartSetiChkBox->setText( i18n( "Invoke SETI@home Client on Startup" ) );
QToolTip::add( StartSetiChkBox, i18n( "Enable this option if you want to start\nthe client together with Ksetiwatch." ) );
ClientLayout->addWidget( StartSetiChkBox );

StopSetiChkBox = new QCheckBox( page, "StopSetiChkBox" );
StopSetiChkBox->setText( i18n( "Stop SETI@home Client on Exit" ) );
QToolTip::add( StopSetiChkBox, i18n( "Enable this option if you want to stop\nthe client together with Ksetiwatch." ) );
ClientLayout->addWidget( StopSetiChkBox );

KeepAliveCheckBox = new QCheckBox( page, "KeepAliveCheckBox" );
KeepAliveCheckBox->setText( i18n( "Keep Client Alive" ) );
QToolTip::add( KeepAliveCheckBox, i18n( "Restart the client if it crashes or stops\nfor unknown reasons." ) );
ClientLayout->addWidget( KeepAliveCheckBox );

ClientSubLayout2 = new QGridLayout( 0, 1, 1, 0, 2, "ClientSubLayout2");

HighLabel = new QLabel( page, "HighLabel" );
HighLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)1, HighLabel->sizePolicy().hasHeightForWidth() ) );
HighLabel->setText( i18n( "High" ) );
HighLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

ClientSubLayout2->addWidget( HighLabel, 2, 2 );

LowLabel = new QLabel( page, "LowLabel" );
LowLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)1, LowLabel->sizePolicy().hasHeightForWidth() ) );
LowLabel->setText( i18n( "Low" ) );

ClientSubLayout2->addWidget( LowLabel, 2, 0 );

PrioritySlider = new QSlider( page, "PrioritySlider" );
PrioritySlider->setMinValue( 1 );
PrioritySlider->setMaxValue( 19 );
PrioritySlider->setLineStep( 1 );
PrioritySlider->setPageStep( 5 );
PrioritySlider->setValue( 1 );
PrioritySlider->setOrientation( QSlider::Horizontal );
PrioritySlider->setTickmarks( QSlider::Right );
PrioritySlider->setTickInterval( 1 );

ClientSubLayout2->addMultiCellWidget( PrioritySlider, 1, 1, 0, 2 );

PriorityLabel = new QLabel( page, "PriorityLabel" );
PriorityLabel->setText( i18n( "Priority:" ) );

ClientSubLayout2->addWidget( PriorityLabel, 0, 0 );
QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
ClientSubLayout2->addItem( spacer, 2, 1 );
ClientLayout->addLayout( ClientSubLayout2 );

ClientSubLayout3 = new QGridLayout( 0, 1, 1, 0, 2, "ClientSubLayout3");

ProxyServerChkBox = new QCheckBox( page, "ProxyServerChkBox" );
ProxyServerChkBox->setText( i18n( "Proxy Server:" ) );

ClientSubLayout3->addWidget( ProxyServerChkBox, 0, 0 );

ProxyServerEdit = new QLineEdit( page, "ProxyServerEdit" );
ProxyServerEdit->setEnabled( FALSE );
ProxyServerEdit->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, ProxyServerEdit->sizePolicy().hasHeightForWidth() ) );

ClientSubLayout3->addWidget( ProxyServerEdit, 1, 0 );

ProxyPortEdit = new QLineEdit( page, "ProxyPortEdit" );
ProxyPortEdit->setEnabled( FALSE );
ProxyPortEdit->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, ProxyPortEdit->sizePolicy().hasHeightForWidth() ) );

ClientSubLayout3->addWidget( ProxyPortEdit, 1, 1 );

ProxyPortLabel = new QLabel( page, "ProxyPortLabel" );
ProxyPortLabel->setEnabled( FALSE );
ProxyPortLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, ProxyPortLabel->sizePolicy().hasHeightForWidth() ) );
ProxyPortLabel->setText( i18n( "Port:" ) );

ClientSubLayout3->addWidget( ProxyPortLabel, 0, 1 );
ClientLayout->addLayout( ClientSubLayout3 );

ClientSubLayout1 = new QGridLayout( 0, 1, 1, 0, 6, "ClientSubLayout1");

ArgEdit = new QLineEdit( page, "ArgEdit" );
QToolTip::add( ArgEdit, i18n( "See the README of the S@h client for details." ) );

ClientSubLayout1->addMultiCellWidget( ArgEdit, 1, 1, 0, 1 );

StopAfterProcChkBox = new QCheckBox( page, "StopAfterProcChkBox" );
StopAfterProcChkBox->setText( i18n( "-stop_after_process" ) );

ClientSubLayout1->addWidget( StopAfterProcChkBox, 2, 0 );

ArgLabel = new QLabel( page, "ArgLabel" );
ArgLabel->setText( i18n( "Command Line Arguments:" ) );

ClientSubLayout1->addMultiCellWidget( ArgLabel, 0, 0, 0, 1 );

GraphicsChkBox = new QCheckBox( page, "GraphicsChkBox" );
GraphicsChkBox->setText( i18n( "-graphics" ) );

ClientSubLayout1->addWidget( GraphicsChkBox, 3, 1 );

StopAfterSendChkBox = new QCheckBox( page, "StopAfterSendChkBox" );
StopAfterSendChkBox->setText( i18n( "-stop_after_send" ) );

ClientSubLayout1->addWidget( StopAfterSendChkBox, 3, 0 );

EmailChkBox = new QCheckBox( page, "EmailChkBox" );
EmailChkBox->setText( i18n( "-email" ) );

ClientSubLayout1->addWidget( EmailChkBox, 2, 1 );
ClientLayout->addLayout( ClientSubLayout1 );
QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
ClientLayout->addItem( spacer_2 );

// signals and slots connections
connect( ProxyServerChkBox, SIGNAL( toggled(bool) ), ProxyServerEdit, SLOT( setEnabled(bool) ) );
connect( ProxyServerChkBox, SIGNAL( toggled(bool) ), ProxyPortLabel, SLOT( setEnabled(bool) ) );
connect( ProxyServerChkBox, SIGNAL( toggled(bool) ), ProxyPortEdit, SLOT( setEnabled(bool) ) );

// tab order
setTabOrder( StartSetiChkBox, StopSetiChkBox );
setTabOrder( StopSetiChkBox, KeepAliveCheckBox );
setTabOrder( KeepAliveCheckBox, PrioritySlider );
setTabOrder( PrioritySlider, ProxyServerChkBox );
setTabOrder( ProxyServerChkBox, ProxyServerEdit );
setTabOrder( ProxyServerEdit, ProxyPortEdit );
setTabOrder( ProxyPortEdit, ArgEdit );
setTabOrder( ArgEdit, StopAfterProcChkBox );
setTabOrder( StopAfterProcChkBox, StopAfterSendChkBox );
setTabOrder( StopAfterSendChkBox, EmailChkBox );
setTabOrder( EmailChkBox, GraphicsChkBox );
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetSetiDirectory()
{
QString d = KFileDialog::getExistingDirectory();
if(!d.isNull())
  setSetiDirectory(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetLogDirectory()
{
QString d = KFileDialog::getExistingDirectory();
if(!d.isNull())
  setLogDirectory(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetStartExe()
{
QString d = KFileDialog::getOpenFileName();
if(!d.isNull())
  setStartExeFilePath(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetStopExe()
{
QString d = KFileDialog::getOpenFileName();
if(!d.isNull())
  setStopExeFilePath(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotHandleText(const QString&)
{
QString l1 = DescrEdit->text();
QString l2 = DirecEdit->text();
if(l1.isEmpty() || l2.isEmpty())
  enableButtonOK(false);
else
  enableButtonOK(true);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotOk()
{
QDir d(DirecEdit->text());
if(d.exists())
  accept();
else
  KMessageBox::error(this, i18n("Directory not found.\n"
                                "Make sure to enter a valid\n"
                                "SETI@Home directory."));
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setOptions(const LocSettings& setloc)
{
// location page
DescrEdit->setText(setloc.description);
ColorButton->setColor(setloc.color);
DirecEdit->setText(setloc.directory);
StartExeChkBox->setChecked(setloc.useCustomStartExe);
StartExeEdit->setText(setloc.customStartExe);
StopExeChkBox->setChecked(setloc.useCustomStopExe);
StopExeEdit->setText(setloc.customStopExe);
// client page
StartSetiChkBox->setChecked(setloc.startSeti);
StopSetiChkBox->setChecked(setloc.stopSeti);
KeepAliveCheckBox->setChecked(setloc.keepClientAlive);
ArgEdit->setText(setloc.arguments);
StopAfterProcChkBox->setChecked(setloc.stopAfterProcess);
StopAfterSendChkBox->setChecked(setloc.stopAfterSend);
EmailChkBox->setChecked(setloc.email);
GraphicsChkBox->setChecked(setloc.graphics);
PrioritySlider->setValue(setloc.priority);
ProxyServerChkBox->setChecked(setloc.useProxy);
ProxyServerEdit->setText(setloc.proxyServer);
ProxyPortEdit->setText(QString::number(setloc.proxyPort));
// logging page
LogChkBox->setChecked(setloc.log);
RedirChkBox->setChecked(setloc.redirectLog);
RedirEdit->setText(setloc.logDirectory);
}

/*------------------------------------------------------------------------ */
LocSettings AddSetiLocDlg::options() const
{
LocSettings setloc;

// location
setloc.description       = DescrEdit->text();
setloc.color             = ColorButton->color();
setloc.directory         = DirecEdit->text();
setloc.useCustomStartExe = StartExeChkBox->isChecked();
setloc.customStartExe    = StartExeEdit->text().stripWhiteSpace();
setloc.useCustomStopExe  = StopExeChkBox->isChecked();
setloc.customStopExe     = StopExeEdit->text().stripWhiteSpace();
// client
setloc.startSeti         = StartSetiChkBox->isChecked();
setloc.stopSeti          = StopSetiChkBox->isChecked();
setloc.keepClientAlive   = KeepAliveCheckBox->isChecked();
setloc.arguments         = ArgEdit->text();
setloc.stopAfterProcess  = StopAfterProcChkBox->isChecked();
setloc.stopAfterSend     = StopAfterSendChkBox->isChecked();
setloc.email             = EmailChkBox->isChecked();
setloc.graphics          = GraphicsChkBox->isChecked();
setloc.priority          = PrioritySlider->value();
setloc.useProxy          = ProxyServerChkBox->isChecked();
setloc.proxyServer       = ProxyServerEdit->text();
setloc.proxyPort         = ProxyPortEdit->text().toInt();
// logging
setloc.log               = LogChkBox->isChecked();
setloc.redirectLog       = RedirChkBox->isChecked();
setloc.logDirectory      = RedirEdit->text().stripWhiteSpace();

return(setloc);
}

#include "addsetilocdlg.moc"

